package com.muyu.pay.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dtflys.forest.Forest;
import com.muyu.domain.OrderPayCustomer;
import com.muyu.domain.OrderPayInfo;
import com.muyu.domain.req.CustomerListReq;
import com.muyu.domain.resp.CustomerResp;
import com.muyu.pay.mapper.OrderPayCustomerMapper;
import com.muyu.pay.service.OrderPayCustomerService;
import com.muyu.pay.service.OrderPayService;
import com.muyu.common.core.enums.SystemYesNo;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.nacos.service.NacosServerService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;


@Log4j2
@Service
public class OrderPayCustomerServiceImpl
        extends ServiceImpl<OrderPayCustomerMapper, OrderPayCustomer>
        implements OrderPayCustomerService {

    @Autowired
    private OrderPayService orderPayService;

    @Autowired
    private NacosServerService nacosServerService;


    @Override
    public List<CustomerResp> selectList (CustomerListReq customerListReq) {
        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(
                StringUtils.isNotEmpty(customerListReq.getAppName()),
                OrderPayCustomer::getAppName, customerListReq.getAppName()
        );
        queryWrapper.like(
                StringUtils.isNotEmpty(customerListReq.getAppCode()),
                OrderPayCustomer::getAppCode, customerListReq.getAppCode()
        );
        queryWrapper.eq(
                StringUtils.isNotEmpty(customerListReq.getStatus()),
                OrderPayCustomer::getStatus, customerListReq.getStatus()
        );
        List<OrderPayCustomer> orderPayCustomerList = this.list(queryWrapper);
        return orderPayCustomerList.stream()
                .map(orderPayCustomer -> CustomerResp.customerBuild(
                                orderPayCustomer,
                                () -> orderPayService.selectOrderPayByAppCodeAndLimit(orderPayCustomer.getAppCode(), 5)
                                        .stream()
                                        .map(OrderPayInfo::buildCustomerOrderPaySimpleResp)
                                        .toList()
                        )
                )
                .toList();
    }


    @Override
    public List<String> getCustomerAllList () {
        List<String> nacosServerAllList = nacosServerService.nacosServiceAllList();
        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select(OrderPayCustomer::getAppCode);
        List<OrderPayCustomer> orderPayCustomerList = this.list(queryWrapper);
        Set<String> customerSet
                = orderPayCustomerList.stream().map(OrderPayCustomer::getAppCode).collect(Collectors.toSet());
        return nacosServerAllList.stream()
                .filter(nacosServer -> !customerSet.contains(nacosServer))
                .toList();
    }

    @Override
    public boolean removeById (Serializable id) {
        OrderPayCustomer orderPayCustomer = this.getById(id);
        if (orderPayCustomer == null) {
            throw new ServiceException("客户不存在");
        }
        LambdaQueryWrapper<OrderPayInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderPayInfo::getAppCode, orderPayCustomer.getAppCode());
        queryWrapper.gt(OrderPayInfo::getCreateTime, DateUtils.addDays(new Date(), -7));
        long count = orderPayService.count(queryWrapper);
        if (count > 0) {
            throw new ServiceException(
                    StringUtils.format("客户：[{}], 七天还用, 不能删除", orderPayCustomer.getAppName())
            );
        }
        return super.removeById(id);
    }


    @Override
    public void disable (Long orderCustomerId) {
        this.settingStatus(orderCustomerId, SystemYesNo.NO.getCode());
    }


    @Override
    public void enable (Long orderCustomerId) {
        this.settingStatus(orderCustomerId, SystemYesNo.YES.getCode());
    }

    public void settingStatus(Long orderCustomerId, String status){
        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderPayCustomer::getId, orderCustomerId);
        boolean isExists = this.exists(queryWrapper);
        if (!isExists){
            throw new ServiceException("操作客户不存在");
        }
        if (!SystemYesNo.isCode(status)){
            throw new ServiceException("设置状态值违法");
        }
        LambdaUpdateWrapper<OrderPayCustomer> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(OrderPayCustomer::getId, orderCustomerId);
        updateWrapper.set(OrderPayCustomer::getStatus, status);
        this.update(updateWrapper);
    }

    @Override
    public boolean save (OrderPayCustomer orderPayCustomer) {
        String appCode = orderPayCustomer.getAppCode();

        List<String> nacosServerAllList = nacosServerService.nacosServiceAllList();
        log.info("进行服务合法性判断：[{}->{}]", appCode, nacosServerAllList);
        if (!nacosServerAllList.contains(appCode)){
            throw new ServiceException("客户编码违法");
        }

        LambdaQueryWrapper<OrderPayCustomer> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(OrderPayCustomer::getAppCode, appCode);
        long isAppCodeOnly = this.count(queryWrapper);
        log.info("进行服务code唯一性校验：[{}-{}]", appCode, isAppCodeOnly);
        if (isAppCodeOnly > 0){
            throw new ServiceException("客户编码重复");
        }

        return super.save(orderPayCustomer);
    }
}
